/*
 * Copyright 2013 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.awk.code;

import net.morilib.awk.io.AwkFiles;
import net.morilib.awk.namespace.AwkNamespace;

/**
 * awkiumのプログラム単位をあらわすインターフェースです。
 * 
 * @author MORIGUCHI, Yuichiro 2013/03
 */
public interface AwkExecutable {

	/**
	 * プログラム単位を実行します。
	 * 
	 * @param ns 名前空間
	 * @param f  ファイル管理オブジェクト
	 * @param s  対象の行
	 */
	public void execute(AwkNamespace ns, AwkFiles f, String s);

	/**
	 * プログラム単位のBEGINアクションを実行します。
	 * 
	 * @param ns 名前空間
	 * @param f  ファイル管理オブジェクト
	 */
	public void executeBegin(AwkNamespace ns, AwkFiles f);

	/**
	 * プログラム単位のENDアクションを実行します。
	 * 
	 * @param ns 名前空間
	 * @param f  ファイル管理オブジェクト
	 */
	public void executeEnd(AwkNamespace ns, AwkFiles f);

	/**
	 * プログラム単位がすべてBEGINであるときにtrueを得ます。
	 */
	public boolean isBegin();

	/**
	 * プログラム単位がすべてENDであるときにtrueを得ます。
	 */
	public boolean isEnd();

}
