/*
 * Copyright 2013 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.sed.cmd;

import java.io.IOException;

import net.morilib.sed.SedCommand;
import net.morilib.sed.SedLineBuffer;
import net.morilib.sed.SedSpace;

/**
 * cコマンドを処理するクラスです。
 *
 *
 * @author MORIGUCHI, Yuichiro 2013/03
 */
public class SedCCommand implements SedCommand {

	private String string;

	/**
	 * コマンドを生成します。
	 * 
	 * @param string 置き換える文字
	 */
	public SedCCommand(String string) {
		this.string = string;
	}

	public String getLongDescription() {
		return "Change";
	}

	public char getName() {
		return 'c';
	}

	public void process(SedSpace s,
			SedLineBuffer rd) throws IOException {
		s.deleteLine();
		s.printImmediate(string);
	}

	/**
	 * 置き換える文字を得ます。
	 * 
	 * @return 置き換える文字
	 */
	public String getString() {
		return string;
	}

	public int hashCode() {
		return string.hashCode();
	}

	public boolean equals(Object o) {
		if(o instanceof SedCCommand) {
			return string.equals(((SedCCommand)o).string);
		}
		return false;
	}

	public String toString() {
		return "c" + string;
	}

}
