/*
 * Copyright 2013 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.sh;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

import net.morilib.sh.misc.PipeBuffer;
import net.morilib.sh.misc.XtraceStream;

public class ShTreePipe implements ShTree {

	private List<ShTree> line;

	public ShTreePipe(List<ShTree> line) {
		this.line = new ArrayList<ShTree>(line);
	}

	public int eval(ShEnvironment env,
			ShFileSystem fs,
			ShBuiltInCommands cmds,
			ShRuntime run,
			InputStream stdin,
			PrintStream stdout,
			PrintStream stderr,
			XtraceStream p) throws IOException, ShSyntaxException {
		PipeBuffer b = null, c = null;
		InputStream in  = stdin;
		PrintStream out;
		int a = 0;
		ShTree t;

		try {
			for(int i = 0; i < line.size(); i++) {
				t   = line.get(i);
				if(i < line.size() - 1) {
					if(c != null)  c.close();
					c   = b;
					b   = new PipeBuffer();
					out = new PrintStream(b.getOutputStream());
				} else {
					out = stdout;
				}
				a = t.eval(env, fs, cmds, run, in, out, stderr, p);
				if(i < line.size() - 1) {
					out.close();
					in  = b.getInputStream();
				}
			}
			return a;
		} finally {
			if(b != null)  b.close();
			if(c != null)  c.close();
		}
	}

	public void compileInternally(ShTreeExpressionMachine.Builder b,
			Object brk, Object cnt) {
		b.add(this);
	}

	public String toString() {
		StringBuffer b = new StringBuffer();
		String d = "";

		for(ShTree l : line) {
			b.append(d).append(l);
			d = " | ";
		}
		return b.toString();
	}

}
